function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function arrayToObject() {
  var fields = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  return fields.reduce(function (prev, path) {
    var key = path.split(".").slice(-1)[0];

    if (prev[key]) {
      throw new Error("The key `".concat(key, "` is already in use."));
    } // eslint-disable-next-line no-param-reassign


    prev[key] = path;
    return prev;
  }, {});
}

function normalizeNamespace(fn) {
  return function (namespace, map, getterType, mutationType) {
    /* eslint-disable no-param-reassign */
    if (typeof namespace !== "string") {
      mutationType = getterType;
      getterType = map;
      map = namespace;
      namespace = "";
    } else if (namespace.charAt(namespace.length - 1) !== "/") {
      namespace += "/";
    }

    getterType = "".concat(namespace).concat(getterType || "getField");
    mutationType = "".concat(namespace).concat(mutationType || "updateField");
    /* eslint-enable */

    return fn(namespace, map, getterType, mutationType);
  };
}

function getField(state) {
  return function (path) {
    return path.split(/[.[\]]+/).reduce(function (prev, key) {
      return prev[key];
    }, state);
  };
}
function updateField(state, _ref) {
  var path = _ref.path,
      value = _ref.value;
  path.split(/[.[\]]+/).reduce(function (prev, key, index, array) {
    if (array.length === index + 1) {
      // eslint-disable-next-line no-param-reassign
      prev[key] = value;
    }

    return prev[key];
  }, state);
}

var _mapFields = normalizeNamespace(function (namespace, fields, getterType, mutationType) {
  var fieldsObject = Array.isArray(fields) ? arrayToObject(fields) : fields;
  return Object.keys(fieldsObject).reduce(function (prev, key) {
    var path = fieldsObject[key];
    var field = {
      get: function get() {
        return this.$store.getters[getterType](path);
      },
      set: function set(value) {
        this.$store.commit(mutationType, {
          path: path,
          value: value
        });
      }
    }; // eslint-disable-next-line no-param-reassign

    prev[key] = field;
    return prev;
  }, {});
});

var _mapMultiRowFields = normalizeNamespace(function (namespace, paths, getterType, mutationType) {
  // export function mapMultiRowFields(paths, getterType = `getField`, mutationType = `updateField`) {
  var pathsObject = Array.isArray(paths) ? arrayToObject(paths) : paths;
  return Object.keys(pathsObject).reduce(function (entries, key) {
    var path = pathsObject[key]; // eslint-disable-next-line no-param-reassign

    entries[key] = {
      get: function get() {
        var store = this.$store;
        var rows = store.getters[getterType](path);
        return rows.map(function (fieldsObject, index) {
          return Object.keys(fieldsObject).reduce(function (prev, fieldKey) {
            var fieldPath = "".concat(path, "[").concat(index, "].").concat(fieldKey);
            return Object.defineProperty(prev, fieldKey, {
              get: function get() {
                return store.getters[getterType](fieldPath);
              },
              set: function set(value) {
                store.commit(mutationType, {
                  path: fieldPath,
                  value: value
                });
              }
            });
          }, {});
        });
      }
    };
    return entries;
  }, {});
});
var createHelpers = function createHelpers(_ref2) {
  var _ref3;

  var getterType = _ref2.getterType,
      mutationType = _ref2.mutationType;
  return _ref3 = {}, _defineProperty(_ref3, getterType, getField), _defineProperty(_ref3, mutationType, updateField), _defineProperty(_ref3, "mapFields", function mapFields(fields) {
    return _mapFields(fields, getterType, mutationType);
  }), _defineProperty(_ref3, "mapMultiRowFields", function mapMultiRowFields(paths) {
    return _mapMultiRowFields(paths, getterType, mutationType);
  }), _ref3;
};

export { getField, updateField, _mapFields as mapFields, _mapMultiRowFields as mapMultiRowFields, createHelpers };
